@@ -1,6 +1,6 @@
-// $Id: yascreen.c,v 1.99 2023/08/02 17:32:12 bbonev Exp $
+// $Id: yascreen.c,v 2.06 2025/01/15 15:57:59 bbonev Exp $
 //
-// Copyright © 2015-2023 Boian Bonev (bbonev@ipacct.com) {{{
+// Copyright © 2015-2025 Boian Bonev (bbonev@ipacct.com) {{{
 //
 // SPDX-License-Identifer: LGPL-3.0-or-later
 //
@@ -265,6 +265,15 @@ static inline void outs(yascreen *s,const char *str) { // {{{
 	}
 } // }}}
 
+static inline int yascreen_vasprintf(yascreen *s,char **ps,const char *format,va_list ap) { // {{{
+	if (!s)
+		return -1;
+	if (!format)
+		return -1;
+
+	return vasprintf(ps,format,ap);
+} // }}}
+
 static inline void outf(yascreen *s,const char *format,...) __attribute__((format(printf,2,3))); // {{{
 // }}}
 
@@ -273,13 +282,8 @@ static inline void outf(yascreen *s,const char *format,...) { // {{{
 	char *ns;
 	int size;
 
-	if (!s)
-		return;
-	if (!format)
-		return;
-
 	va_start(ap,format);
-	size=vasprintf(&ns,format,ap);
+	size=yascreen_vasprintf(s,&ns,format,ap);
 	va_end(ap);
 
 	if (size==-1) // some error, nothing more to do
@@ -314,13 +318,77 @@ inline void *yascreen_get_hint_p(yascreen *s) { // {{{
 	return s->phint;
 } // }}}
 
-static char myver[]="\0Yet another screen library (https://github.com/bbonev/yascreen) $Revision: 1.99 $\n\n"; // {{{
+static char myver[]="\0Yet another screen library (https://github.com/bbonev/yascreen) $Revision: 2.06 $\n\n"; // {{{
 // }}}
 
 inline const char *yascreen_ver(void) { // {{{
 	return myver;
 } // }}}
 
+static inline void yascreen_auto_size(yascreen *s,int *sx,int *sy) { // {{{
+	if (!sx||!sy)
+		return;
+
+	if (!*sx||!*sy)
+		if (!s->outcb&&isatty(STDOUT_FILENO)) {
+			struct winsize ws={0};
+
+			if (!ioctl(STDOUT_FILENO,TIOCGWINSZ,&ws)) {
+				if (!ws.ws_col)
+					ws.ws_col=80;
+				if (!ws.ws_row)
+					ws.ws_row=24;
+				if (!*sx)
+					*sx=ws.ws_col;
+				if (!*sy)
+					*sy=ws.ws_row;
+			} else {
+				if (!*sx)
+					*sx=80;
+				if (!*sy)
+					*sy=24;
+			}
+		}
+} // }}}
+
+static inline void yascreen_free_memcell(yascreen *s,int i) { // {{{
+	if (s->mem[i].style&YAS_STORAGE) {
+		free(s->mem[i].p);
+		s->mem[i].style&=~YAS_STORAGE;
+	}
+} // }}}
+
+static inline void yascreen_free_cell(yascreen *s,int i) { // {{{
+	yascreen_free_memcell(s,i);
+	if (s->scr[i].style&YAS_STORAGE) {
+		free(s->scr[i].p);
+		s->scr[i].style&=~YAS_STORAGE;
+	}
+} // }}}
+
+static inline void yascreen_empty_cell(yascreen *s,int i) { // {{{
+	s->mem[i].style=s->scr[i].style=0;
+	s->mem[i].d[0]=' ';
+	s->mem[i].d[1]=0;
+	s->scr[i].d[0]=0;
+} // }}}
+
+static inline void yascreen_free_empty_cell(yascreen *s,int i) { // {{{
+	yascreen_free_cell(s,i);
+	yascreen_empty_cell(s,i);
+} // }}}
+
+static inline void yascreen_free_dynamic(yascreen *s) { // {{{
+	if (s->mem)
+		free(s->mem);
+	if (s->scr)
+		free(s->scr);
+	if (s->tsstack)
+		free(s->tsstack);
+	if (s->keys)
+		free(s->keys);
+} // }}}
+
 inline yascreen *yascreen_init(int sx,int sy) { // {{{
 	yascreen *s;
 	int i;
@@ -353,20 +421,11 @@ inline yascreen *yascreen_init(int sx,int sy) { // {{{
 		}
 		s->tssize=1;
 		tcgetattr(STDOUT_FILENO,s->tsstack);
-		if (!sx||!sy) {
-			struct winsize ws={0};
 
-			if (!ioctl(STDOUT_FILENO,TIOCGWINSZ,&ws)) {
-				if (!sx)
-					sx=ws.ws_col;
-				if (!sy)
-					sy=ws.ws_row;
-			}
-		}
+		yascreen_auto_size(s,&sx,&sy);
 	}
 	if (sx<=0||sy<=0) {
-		if (s->tsstack)
-			free(s->tsstack);
+		yascreen_free_dynamic(s);
 		free(s);
 		return NULL;
 	}
@@ -408,8 +467,7 @@ inline yascreen *yascreen_init(int sx,int sy) { // {{{
 
 	s->keys=(int *)calloc(KEYSTEP,sizeof(int));
 	if (!s->keys) {
-		if (s->tsstack)
-			free(s->tsstack);
+		yascreen_free_dynamic(s);
 		free(s);
 		return NULL;
 	}
@@ -417,14 +475,7 @@ inline yascreen *yascreen_init(int sx,int sy) { // {{{
 	s->mem=(cell *)calloc(sx*sy,sizeof(cell));
 	s->scr=(cell *)calloc(sx*sy,sizeof(cell));
 	if (!s->mem||!s->scr) {
-		if (s->mem)
-			free(s->mem);
-		if (s->scr)
-			free(s->scr);
-		if (s->tsstack)
-			free(s->tsstack);
-		if (s->keys)
-			free(s->keys);
+		yascreen_free_dynamic(s);
 		free(s);
 		return NULL;
 	}
@@ -490,17 +541,7 @@ inline int yascreen_resize(yascreen *s,int sx,int sy) { // {{{
 	if (sx<0||sy<0)
 		return -1;
 
-	if (!sx||!sy)
-		if (!s->outcb&&isatty(STDOUT_FILENO)) {
-			struct winsize ws={0};
-
-			if (!ioctl(STDOUT_FILENO,TIOCGWINSZ,&ws)) {
-				if (!sx)
-					sx=ws.ws_col;
-				if (!sy)
-					sy=ws.ws_row;
-			}
-		}
+	yascreen_auto_size(s,&sx,&sy);
 
 	if (sx<=0||sy<=0)
 		return -1;
@@ -508,17 +549,11 @@ inline int yascreen_resize(yascreen *s,int sx,int sy) { // {{{
 	if (s->sx==sx&&s->sy==sy)
 		return 0;
 
-	for (i=0;i<s->sx*s->sy;i++) { // free old allocated data and set for reusage
-		if (s->mem[i].style&YAS_STORAGE)
-			free(s->mem[i].p);
-		if (s->scr[i].style&YAS_STORAGE)
-			free(s->scr[i].p);
-		if (i<sx*sy) {
-			s->mem[i].style=s->scr[i].style=0;
-			s->mem[i].d[0]=' ';
-			s->scr[i].d[0]=0;
-		}
-	}
+	for (i=0;i<s->sx*s->sy;i++) // free old allocated data and set for reusage
+		if (i<sx*sy)
+			yascreen_free_empty_cell(s,i);
+		else
+			yascreen_free_cell(s,i);
 	if (sx*sy>s->sx*s->sy) { // allocate bigger buffer
 		mem=(cell *)realloc(s->mem,sx*sy*sizeof(cell));
 		if (!mem)
@@ -528,11 +563,8 @@ inline int yascreen_resize(yascreen *s,int sx,int sy) { // {{{
 		if (!scr)
 			return -1;
 		s->scr=scr;
-		for (i=s->sx*s->sy;i<sx*sy;i++) { // initialize the rest of the area
-			s->mem[i].style=s->scr[i].style=0;
-			s->mem[i].d[0]=' ';
-			s->scr[i].d[0]=0;
-		}
+		for (i=s->sx*s->sy;i<sx*sy;i++) // initialize the rest of the area
+			yascreen_empty_cell(s,i);
 	}
 	s->redraw=1;
 	s->sx=sx;
@@ -548,29 +580,13 @@ inline void yascreen_free(yascreen *s) { // {{{
 		return;
 
 	if (!s->mem||!s->scr) { // error condition that will happen only if mem is corrupt
-		if (s->mem)
-			free(s->mem);
-		if (s->scr)
-			free(s->scr);
-		if (s->tsstack)
-			free(s->tsstack);
-		if (s->keys)
-			free(s->keys);
+		yascreen_free_dynamic(s);
 		free(s); // most probably will crash, because there is no way to have s partally initialized
 		return;
 	}
-	for (i=0;i<s->sx*s->sy;i++) {
-		if (s->mem[i].style&YAS_STORAGE)
-			free(s->mem[i].p);
-		if (s->scr[i].style&YAS_STORAGE)
-			free(s->scr[i].p);
-	}
-	free(s->mem);
-	free(s->scr);
-	if (s->tsstack)
-		free(s->tsstack);
-	if (s->keys)
-		free(s->keys);
+	for (i=0;i<s->sx*s->sy;i++)
+		yascreen_free_cell(s,i);
+	yascreen_free_dynamic(s);
 	outs(s,ESC"[0m");
 	free(s);
 } // }}}
@@ -718,11 +734,7 @@ static inline void yascreen_putcw(yascreen *s,uint32_t attr,const char *str,int
 		int i;
 
 		// normal char
-		if (s->mem[s->cursorx+s->cursory*s->sx].style&YAS_STORAGE) {
-			s->mem[s->cursorx+s->cursory*s->sx].style&=~YAS_STORAGE;
-			free(s->mem[s->cursorx+s->cursory*s->sx].p);
-			s->mem[s->cursorx+s->cursory*s->sx].p=0;
-		}
+		yascreen_free_memcell(s,s->cursorx+s->cursory*s->sx);
 		if (strlen(str)<PSIZE) {
 			strncpy(s->mem[s->cursorx+s->cursory*s->sx].d,str,sizeof s->mem[s->cursorx+s->cursory*s->sx].d);
 			s->mem[s->cursorx+s->cursory*s->sx].style=attr;
@@ -740,11 +752,7 @@ static inline void yascreen_putcw(yascreen *s,uint32_t attr,const char *str,int
 		s->cursorx++;
 		for (i=1;i<width;i++) {
 			if (s->cursorx<s->sx) {
-				if (s->mem[s->cursorx+s->cursory*s->sx].style&YAS_STORAGE) {
-					s->mem[s->cursorx+s->cursory*s->sx].style&=~YAS_STORAGE;
-					free(s->mem[s->cursorx+s->cursory*s->sx].p);
-					s->mem[s->cursorx+s->cursory*s->sx].p=0;
-				}
+				yascreen_free_memcell(s,s->cursorx+s->cursory*s->sx);
 				*s->mem[s->cursorx+s->cursory*s->sx].d=0;
 				s->mem[s->cursorx+s->cursory*s->sx].style=attr;
 			}
@@ -756,11 +764,7 @@ static inline void yascreen_putcw(yascreen *s,uint32_t attr,const char *str,int
 		int x;
 
 		for (x=0;x<s->cursorx+width;x++) { // zap spanned chars
-			if (s->mem[x+s->cursory*s->sx].style&YAS_STORAGE) {
-				s->mem[x+s->cursory*s->sx].style&=~YAS_STORAGE;
-				free(s->mem[x+s->cursory*s->sx].p);
-				s->mem[x+s->cursory*s->sx].p=0;
-			}
+			yascreen_free_memcell(s,x+s->cursory*s->sx);
 			strncpy(s->mem[x+s->cursory*s->sx].d,"<",sizeof s->mem[x+s->cursory*s->sx].d);
 			s->mem[x+s->cursory*s->sx].style=attr;
 			s->mem[s->cursory*s->sx].style|=YAS_TOUCHED;
@@ -808,11 +812,7 @@ static inline void yascreen_putcw(yascreen *s,uint32_t attr,const char *str,int
 		int x;
 
 		for (x=s->cursorx;x<s->sx;x++) { // zap spanned chars
-			if (s->mem[x+s->cursory*s->sx].style&YAS_STORAGE) {
-				s->mem[x+s->cursory*s->sx].style&=~YAS_STORAGE;
-				free(s->mem[x+s->cursory*s->sx].p);
-				s->mem[x+s->cursory*s->sx].p=0;
-			}
+			yascreen_free_memcell(s,x+s->cursory*s->sx);
 			strncpy(s->mem[x+s->cursory*s->sx].d,">",sizeof s->mem[x+s->cursory*s->sx].d);
 			s->mem[x+s->cursory*s->sx].style=attr;
 			s->mem[s->cursory*s->sx].style|=YAS_TOUCHED;
@@ -1131,13 +1131,8 @@ inline int yascreen_printxy(yascreen *s,int x,int y,uint32_t attr,const char *fo
 	int size;
 	int rv;
 
-	if (!s)
-		return -1;
-	if (!format)
-		return -1;
-
 	va_start(ap,format);
-	size=vasprintf(&ns,format,ap);
+	size=yascreen_vasprintf(s,&ns,format,ap);
 	va_end(ap);
 
 	if (size==-1) // some error, nothing more to do
@@ -1155,13 +1150,8 @@ inline int yascreen_printxyu(yascreen *s,int x,int y,uint32_t attr,const char *f
 	int size;
 	int rv;
 
-	if (!s)
-		return -1;
-	if (!format)
-		return -1;
-
 	va_start(ap,format);
-	size=vasprintf(&ns,format,ap);
+	size=yascreen_vasprintf(s,&ns,format,ap);
 	va_end(ap);
 
 	if (size==-1) // some error, nothing more to do
@@ -1207,13 +1197,8 @@ inline int yascreen_print(yascreen *s,const char *format,...) { // {{{
 	int size;
 	int rv;
 
-	if (!s)
-		return -1;
-	if (!format)
-		return -1;
-
 	va_start(ap,format);
-	size=vasprintf(&ns,format,ap);
+	size=yascreen_vasprintf(s,&ns,format,ap);
 	va_end(ap);
 
 	if (size==-1) // some error, nothing more to do
@@ -1569,7 +1554,7 @@ inline void yascreen_feed_179(yascreen *s,unsigned char c);
 #endif
 
 static inline int yascreen_getch_to_gen(yascreen *s,int timeout,int key_none) { // {{{
-	int64_t toms=timeout*1000,tto;
+	int64_t toms=timeout*1000L,tto;
 	struct timeval to,*pto=&to;
 	fd_set r;
 
@@ -1645,21 +1630,30 @@ inline int V(yascreen_getch_to,V193)(yascreen *s,int timeout) {
 	return yascreen_getch_to_gen(s,timeout,YAS_K_NONE);
 } // }}}
 
-inline void yascreen_ungetch(yascreen *s,int key) { // {{{
+static inline int yascreen_realloc_keys(yascreen *s) { // {{{
 	int *tk;
 
 	if (!s)
-		return;
+		return 1;
 
 	if (s->keysize<=s->keycnt) { // need to reallocate key storage
 		int newsize=s->keysize+KEYSTEP;
 
 		tk=(int *)realloc(s->keys,sizeof(int)*newsize);
 		if (!tk)
-			return;
+			return 1;
 		s->keys=tk;
 		s->keysize=newsize;
 	}
+	return 0;
+} // }}}
+
+inline void yascreen_ungetch(yascreen *s,int key) { // {{{
+	if (!s)
+		return;
+
+	if (yascreen_realloc_keys(s))
+		return;
 
 	memmove(s->keys+1,s->keys,sizeof(int)*s->keycnt);
 	s->keys[0]=key;
@@ -1667,20 +1661,11 @@ inline void yascreen_ungetch(yascreen *s,int key) { // {{{
 } // }}}
 
 inline void yascreen_pushch(yascreen *s,int key) { // {{{
-	int *tk;
-
 	if (!s)
 		return;
 
-	if (s->keysize<=s->keycnt) { // need to reallocate key storage
-		int newsize=s->keysize+KEYSTEP;
-
-		tk=(int *)realloc(s->keys,sizeof(int)*newsize);
-		if (!tk)
-			return;
-		s->keys=tk;
-		s->keysize=newsize;
-	}
+	if (yascreen_realloc_keys(s))
+		return;
 
 	s->keys[s->keycnt++]=key;
 } // }}}
